import { Server } from 'ws';
import crypto from 'crypto';

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var base64id = createCommonjsModule(function (module, exports) {
/*!
 * base64id v0.1.0
 */

/**
 * Module dependencies
 */



/**
 * Constructor
 */

var Base64Id = function() { };

/**
 * Get random bytes
 *
 * Uses a buffer if available, falls back to crypto.randomBytes
 */

Base64Id.prototype.getRandomBytes = function(bytes) {

  var BUFFER_SIZE = 4096;
  var self = this;  
  
  bytes = bytes || 12;

  if (bytes > BUFFER_SIZE) {
    return crypto.randomBytes(bytes);
  }
  
  var bytesInBuffer = parseInt(BUFFER_SIZE/bytes);
  var threshold = parseInt(bytesInBuffer*0.85);

  if (!threshold) {
    return crypto.randomBytes(bytes);
  }

  if (this.bytesBufferIndex == null) {
     this.bytesBufferIndex = -1;
  }

  if (this.bytesBufferIndex == bytesInBuffer) {
    this.bytesBuffer = null;
    this.bytesBufferIndex = -1;
  }

  // No buffered bytes available or index above threshold
  if (this.bytesBufferIndex == -1 || this.bytesBufferIndex > threshold) {
     
    if (!this.isGeneratingBytes) {
      this.isGeneratingBytes = true;
      crypto.randomBytes(BUFFER_SIZE, function(err, bytes) {
        self.bytesBuffer = bytes;
        self.bytesBufferIndex = 0;
        self.isGeneratingBytes = false;
      }); 
    }
    
    // Fall back to sync call when no buffered bytes are available
    if (this.bytesBufferIndex == -1) {
      return crypto.randomBytes(bytes);
    }
  }
  
  var result = this.bytesBuffer.slice(bytes*this.bytesBufferIndex, bytes*(this.bytesBufferIndex+1)); 
  this.bytesBufferIndex++; 
  
  return result;
};

/**
 * Generates a base64 id
 *
 * (Original version from socket.io <http://socket.io>)
 */

Base64Id.prototype.generateId = function () {
  var rand = new Buffer(15); // multiple of 3 for base64
  if (!rand.writeInt32BE) {
    return Math.abs(Math.random() * Math.random() * Date.now() | 0).toString()
      + Math.abs(Math.random() * Math.random() * Date.now() | 0).toString();
  }
  this.sequenceNumber = (this.sequenceNumber + 1) | 0;
  rand.writeInt32BE(this.sequenceNumber, 11);
  if (crypto.randomBytes) {
    this.getRandomBytes(12).copy(rand);
  } else {
    // not secure for node 0.4
    [0, 4, 8].forEach(function(i) {
      rand.writeInt32BE(Math.random() * Math.pow(2, 32) | 0, i);
    });
  }
  return rand.toString('base64').replace(/\//g, '_').replace(/\+/g, '-');
};

/**
 * Export
 */

exports = module.exports = new Base64Id();
});

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */













function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var Socket = /** @class */ (function () {
    function Socket(ws, request, listeners, middleware) {
        this.ws = ws;
        this.request = request;
        this.middleware = middleware;
        this.update(listeners, middleware);
    }
    Socket.prototype.update = function (listeners, middleware) {
        var _this = this;
        this.ws.removeAllListeners();
        this.middleware = middleware;
        listeners.forEach(function (handlers, event) {
            if (event === 'connection') {
                return;
            }
            handlers.forEach(function (handler) {
                _this.on(event, handler);
            });
        });
    };
    /**
     * Adds a specific event and callback to this socket
     */
    Socket.prototype.on = function (event, handler) {
        var _this = this;
        this.ws.on(event, function (message) {
            var ctx = {
                event: event,
                data: message,
                request: _this.request,
                socket: _this.ws,
            };
            _this.invokeHandler(handler, ctx, message);
        });
    };
    Socket.prototype.invokeHandler = function (handler, ctx, message) {
        var _this = this;
        if (!this.middleware) {
            handler(ctx, message);
            return;
        }
        this.middleware(ctx, function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                handler(ctx, message);
                return [2 /*return*/];
            });
        }); });
    };
    return Socket;
}());

var compose = require('koa-compose');
var WsSocket = /** @class */ (function () {
    function WsSocket() {
        var _this = this;
        this.middleware = [];
        this.composed = null;
        this.connections = new Map();
        this.listeners = new Map();
        this.wss = null;
        this.onConnection = function (ws, request) {
            var socketId = base64id.generateId();
            var socketInstance = new Socket(ws, request, _this.listeners, _this.composed);
            _this.connections.set(socketId, socketInstance);
            ws.on('disconnect', function () {
                _this.connections.delete(socketId);
            });
            var handlers = _this.listeners.get('connection');
            if (handlers) {
                handlers.forEach(function (handler) {
                    handler({
                        event: 'connection',
                        data: socketInstance,
                        request: request,
                        socket: ws,
                    }, null);
                });
            }
        };
    }
    WsSocket.prototype.attach = function (app, https, tlsOpts) {
        if (https === void 0) { https = false; }
        if (tlsOpts === void 0) { tlsOpts = {}; }
        var enhancedApp = app;
        if (enhancedApp.server &&
            enhancedApp.server.constructor.name !== 'Server') {
            throw new Error("app.server already exists but it's not an http server");
        }
        if (!enhancedApp.server) {
            // Create a server if it doesn't already exists
            enhancedApp.server = https
                ? require('https').createServer(tlsOpts, enhancedApp.callback())
                : require('http').createServer(enhancedApp.callback());
            // Patch `app.listen()` to call `app.server.listen()`
            enhancedApp.listen = function listen() {
                enhancedApp.server.listen.apply(enhancedApp.server, arguments);
                return enhancedApp.server;
            };
        }
        this.wss = new Server({ server: enhancedApp.server });
        enhancedApp.ws = this;
        this.wss.on('connection', this.onConnection);
    };
    /**
     * Pushes a middleware on to the stack
     * @param fn <Function> the middleware function to execute
     */
    WsSocket.prototype.use = function (fn) {
        this.middleware.push(fn);
        this.composed = compose(this.middleware);
        this.updateConnections();
        return this;
    };
    /**
     * Adds a new listener to the stack
     * @param event <String> the event id
     * @param handler <Function> the callback to execute
     * @return this
     */
    WsSocket.prototype.on = function (event, handler) {
        var listeners = this.listeners.get(event) || [];
        listeners.push(handler);
        this.listeners.set(event, listeners);
        this.updateConnections();
        return this;
    };
    /**
     * Removes a listener from the event
     * @param event <String> if omitted will remove all listeners
     * @param handler <Function> if omitted will remove all from the event
     * @return this
     */
    WsSocket.prototype.off = function (event, handler) {
        if (!event) {
            this.listeners = new Map();
            this.updateConnections();
            return this;
        }
        if (!handler) {
            this.listeners.delete(event);
            this.updateConnections();
            return this;
        }
        var listeners = this.listeners.get(event) || [];
        var i = listeners.length - 1;
        while (i) {
            if (listeners[i] === handler) {
                break;
            }
            i--;
        }
        listeners.splice(i, 1);
        this.updateConnections();
        return this;
    };
    /**
     * Updates all existing connections with current listeners and middleware
     * @private
     */
    WsSocket.prototype.updateConnections = function () {
        var _this = this;
        this.connections.forEach(function (connection) {
            return connection.update(_this.listeners, _this.composed);
        });
    };
    return WsSocket;
}());

export { WsSocket };
//# sourceMappingURL=koa-ws-socket.es5.js.map
